"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.httpRequestSerializers = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _http = _interopRequireDefault(require("http"));

var _bunyan = _interopRequireDefault(require("bunyan"));

var _bunyanDebugStream = _interopRequireDefault(require("bunyan-debug-stream"));

var _loggingBunyan = require("@google-cloud/logging-bunyan");

/*
 * convert a millisecond value into a 'Duration' object, defined as
 * `{ seconds, nanos }`, where `nanos` is the nanosecond component of the
 * time
 *
 * https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#google.protobuf.Duration
 */
var formatDuration = function formatDuration(ms) {
  var seconds = Math.floor(ms / 1000); // whole seconds

  var nanos = ms % 1000 * 1000 * 1000; // remainder milliseconds in nanoseconds (= ms * 1,000,000)

  return {
    seconds: seconds,
    nanos: nanos
  };
};

var serializers = (0, _objectSpread2.default)({}, _bunyan.default.stdSerializers);
/*
 * These serializers correspond to the 2 major types of 'request'/'response'
 * objects that are used in MWP apps - Hapi requests and http.IncomingMessage
 * responses that are returned from the `request` library for server-to-server
 * requests.
 */

var httpRequestSerializers = {
  hapi: function hapi(request) {
    var requestInfo = {
      requestMethod: request.method.toUpperCase(),
      requestUrl: request.url.href,
      requestSize: request.headers['content-length'],
      userAgent: request.headers['user-agent'],
      remoteIp: request.info.remoteAddress,
      serverIp: request.server.info.address,
      // internal IP
      referer: request.info.referrer
    };

    if (request.response) {
      requestInfo.responseSize = request.response.headers['content-length'];
      requestInfo.status = request.response.statusCode;
      requestInfo.latency = formatDuration(request.info.completed - request.info.received);
    }

    return requestInfo;
  },
  incomingMessage: function incomingMessage(message) {
    var messageInfo = {
      requestMethod: message.request.method.toUpperCase(),
      requestUrl: message.request.uri.href,
      responseSize: (message.body || '').length,
      status: message.statusCode,
      latency: message.elapsedTime && formatDuration(message.elapsedTime),
      userAgent: message.request.headers['user-agent'],
      remoteIp: message.request.headers['x_forwarded_for'] || message.request.headers['remote_addr'],
      referer: message.request.headers['referer']
    };
    return messageInfo;
  }
};
/*
 * Parse an http.IncomingMessage instance from `request` into an error `context`
 * object defined by https://cloud.google.com/error-reporting/docs/formatting-error-messages
 */

exports.httpRequestSerializers = httpRequestSerializers;
var errorContextSerializers = {
  hapi: function hapi(request) {
    var context = {
      httpRequest: {
        method: request.method.toUpperCase(),
        url: request.url.href,
        userAgent: request.headers['user-agent'],
        referrer: request.headers['referer'],
        responseStatusCode: (request.response || {}).statusCode || 500,
        remoteIp: request.headers['x_forwarded_for'] || request.headers['remote_addr']
      }
    };
    var memberId = request.headers['x-member'];

    if (memberId && memberId !== '0') {
      // Stackdriver Error Reporting specifically does not want this field for
      // logged-out users
      context.user = memberId;
    }

    return context;
  },
  incomingMessage: function incomingMessage(response) {
    return {
      httpRequest: {
        method: response.request.method.toUpperCase(),
        url: response.request.uri.href,
        userAgent: response.request.headers['user-agent'],
        referrer: response.request.headers['referer'],
        responseStatusCode: response.statusCode,
        remoteIp: response.request.headers['x_forwarded_for'] || response.request.headers['remote_addr']
      }
    };
  },
  requestOpts: function requestOpts(_requestOpts) {
    return {
      httpRequest: {
        method: _requestOpts.method.toUpperCase(),
        url: _requestOpts.url,
        userAgent: _requestOpts.headers['user-agent'],
        referrer: _requestOpts.headers['referer'],
        responseStatusCode: 500,
        // only used when logging request errors
        remoteIp: _requestOpts.headers['x_forwarded_for'] || _requestOpts.headers['remote_addr']
      }
    };
  }
};

serializers.context = function (request) {
  var hapi = errorContextSerializers.hapi,
      incomingMessage = errorContextSerializers.incomingMessage,
      requestOpts = errorContextSerializers.requestOpts; // choose the serializer based on the type of the object

  if (request.server) {
    // assume a Hapi request
    return hapi(request);
  }

  if (request instanceof _http.default.IncomingMessage) {
    return incomingMessage(request);
  }

  if (request.agentOptions) {
    return requestOpts(request);
  }

  return request;
};
/*
 * Format a Hapi request object as a Stackdriver httpRequest for pretty logging
 * https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry#httprequest
 */


serializers.httpRequest = function (request) {
  var hapi = httpRequestSerializers.hapi,
      incomingMessage = httpRequestSerializers.incomingMessage; // choose the serializer based on the type of the object

  if (request.server) {
    // assume a Hapi request
    return hapi(request);
  }

  if (request instanceof _http.default.IncomingMessage) {
    return incomingMessage(request);
  }

  logger.warn(request, 'Unsupported httpRequest object');
  return {};
};

var streams = [];
var _process$env = process.env,
    NODE_ENV = _process$env.NODE_ENV,
    GAE_INSTANCE = _process$env.GAE_INSTANCE,
    GCLOUD_PROJECT = _process$env.GCLOUD_PROJECT,
    GAE_VERSION = _process$env.GAE_VERSION,
    GAE_SERVICE = _process$env.GAE_SERVICE,
    DISABLE_GAE_LOG = _process$env.DISABLE_GAE_LOG;

if (!NODE_ENV || NODE_ENV === 'development') {
  streams.push({
    type: 'raw',
    stream: (0, _bunyanDebugStream.default)({
      forceColor: true,
      basepath: process.cwd()
    })
  });
}

if (GAE_INSTANCE && !DISABLE_GAE_LOG) {
  var GAELogger = new _loggingBunyan.LoggingBunyan({
    logName: 'mwp_log',
    resource: {
      type: 'gae_app',
      labels: {
        project_id: GCLOUD_PROJECT,
        module_id: GAE_SERVICE,
        version_id: GAE_VERSION
      }
    },
    serviceContext: {
      service: GAE_SERVICE,
      version: GAE_VERSION
    }
  });
  GAELogger.on('error', function (err) {
    return console.error(err);
  });
  streams.push(GAELogger.stream());
}

if (streams.length === 0 && (NODE_ENV !== 'test' || process.argv.includes('--verbose'))) {
  // fallback if no stream specified
  streams.push({
    stream: process.stdout
  });
}

var logger = _bunyan.default.createLogger({
  name: 'mwp-logger',
  serializers: serializers,
  streams: streams
});

if (GAE_INSTANCE && !DISABLE_GAE_LOG) {
  logger.info({
    GAE_SERVICE: GAE_SERVICE,
    GAE_VERSION: GAE_VERSION,
    GAE_INSTANCE: GAE_INSTANCE
  }, 'GAE logging initialized');
}

var _default = logger;
exports.default = _default;
//# sourceMappingURL=logger.js.map